unit ComplexUnit;

interface

type
  TComplex = record
    Re, Img: Double;
  end;

const EPSILON = 0.00001;

function CompAdd(C1, C2: TComplex): TComplex;
function CompSub(C1, C2: TComplex): TComplex;
function CompMul(C1, C2: TComplex): TComplex;
function CompDiv(C1, C2: TComplex): TComplex;

function CompDouble(C: TComplex): TComplex;
function CompExp(C: TComplex; Degree: Integer): TComplex;

implementation

function CompAdd(C1, C2: TComplex): TComplex;
begin
  Result.Re := C1.Re + C2.Re;
  Result.Img := C1.Img + C2.Img;
end;

function CompSub(C1, C2: TComplex): TComplex;
begin
  Result.Re := C1.Re - C2.Re;
  Result.Img := C1.Img - C2.Img;
end;

function CompMul(C1, C2: TComplex): TComplex;
begin
  Result.Re := C1.Re * C2.Re - C1.Img * C2.Img;
  Result.Img := C1.Re * C2.Img + C1.Img * C2.Re;
end;

function CompDiv(C1, C2: TComplex): TComplex;
var
  Temp:  Double;
begin
  Result.Re := C1.Re * C2.Re + C1.Img * C2.Img;
  Result.Img := C1.Img * C2.Re - C1.Re * C2.Img;
  Temp := C2.Re * C2.Re + C2.Img * C2.Img;
  if Temp = 0 then
    Temp := EPSILON;
  Result.Re := Result.Re / Temp;
  Result.Img := Result.Img / Temp;
end;

function CompDouble(C: TComplex): TComplex;
begin
  Result := CompMul(C, C);
end;

function CompExp(C: TComplex; Degree: Integer): TComplex;
begin

  if Degree = 0 then
  begin
    Result.Re := 1;
    Result.Img := 0;
  end
  else if Degree = 1 then
    Result := C
  else if (Degree and 1) = 0 then
    Result := CompDouble(CompExp(C, Degree shr 1))
  else
    Result := CompMul(C, CompExp(C, Degree - 1));
end;

end.
