unit Fractal1Unit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls;

type
  TRange = record
    head: Double;
    tail: Double;
  end;
  PRange = ^TRange;

  TRangeRecurseFunc = procedure(R: TRange; var L:TList);

  TFractal1 = class(TFrame)
    ImgAll: TImage;
  private
    FRangeList: TList;
    FStartRange: TRange;
    FViewRange: TRange;
    FDepth: Integer;
    FRecurseFunc: TRangeRecurseFunc;

    procedure SetDepth(ADepth: Integer);
    procedure SetStartRange(ARange: TRange);
    procedure SetViewRange(ARange: TRange);

    procedure Init;
    procedure FreeAll;
    procedure Recurse;
    { Private declarations }
  public
    property RecurseFunc: TRangeRecurseFunc read FRecurseFunc write FRecurseFunc;
    property Depth:Integer read FDepth write SetDepth;
    property StartRange: TRange read FStartRange write SetStartRange;
    property ViewRange: TRange read FViewRange write SetViewRange;
    procedure ShowRangeList;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    { Public declarations }
  end;

implementation

{$R *.DFM}
procedure TFractal1.ShowRangeList;
  function ViewTrans(X: Double): Integer;
  begin
    Result := Round((X - ViewRange.head) / (ViewRange.tail - ViewRange.head) * ImgAll.Width);
  end;
var
  P: PRange;
  I: Integer;
begin
  ImgAll.Canvas.Brush.Color := clWhite;
  ImgAll.Canvas.FillRect(RECT(0, 0, ImgAll.Width, ImgAll.Height));
  ImgAll.Canvas.TextOut(5, 5+ImgAll.Height div 2, FloatToStr(ViewRange.head));
  ImgAll.Canvas.TextOut(ImgAll.Width -5 - ImgAll.Canvas.TextWidth(FloatToStr(ViewRange.tail)), 5+ImgAll.Height div 2, FloatToStr(ViewRange.tail));

  for I:= 0 to FRangeList.Count - 1 do
  begin
    P := PRange(FRangeList.Items[I]);
    ImgAll.Canvas.MoveTo(ViewTrans(P^.head), ImgAll.Height div 2);
    ImgAll.Canvas.LineTo(ViewTrans(P^.tail), ImgAll.Height div 2);
  end;
end;

procedure TFractal1.Recurse;
var
  NewList, ResList: TList;
  P: PRange;
  I, J: Integer;
begin
  NewList := TList.Create;
  ResList := TList.Create;

  for I:= 0 to FRangeList.Count-1 do
  begin
    P := PRange(FRangeList.Items[I]);
    RecurseFunc(P^, ResList);
    for J:= 0 to ResList.Count-1 do
      NewList.Add(ResList.Items[J]);
    ResList.Clear;
  end;
  ResList.Free;
  FreeAll;
  FRangeList := NewList;
end;

procedure TFractal1.Init;
var
  P: PRange;
begin
  FRangeList := TList.Create;
  New(P);
  P^.head := FStartRange.head;
  P^.tail := FStartRange.tail;
  FRangeList.Add(P);
end;

procedure DefFunc(R: TRange; var L: TList);
var
  P: PRange;
begin
  New(P);
  P^.head := r.head / 3;
  P^.tail := r.tail / 3;
  l.add(P);
  New(P);
  P^.head := r.head / 3 + 2/3;
  P^.tail := r.tail / 3 + 2/3;
  l.add(P);
end;

constructor TFractal1.Create(AOwner: TComponent);
begin
  inherited;
  FStartRange.head := 0;
  FStartRange.tail := 1;
  FViewRange.head := 0;
  FViewRange.tail := 1;
  FDepth := 0;
  Init;
  RecurseFunc := DefFunc;
end;

procedure TFractal1.FreeAll;
begin
  while FRangeList.Count > 0 do
  begin
    Dispose(PRange(FRangeList.Last));
    FRangeList.Delete(FRangeList.Count - 1);
  end;
  FRangeList.Free;
end;

destructor TFractal1.Destroy;
begin
  FreeAll;
  inherited;
end;

procedure TFractal1.SetDepth(ADepth: Integer);
var
  I: Integer;
begin
  if ADepth < 10 then
    FDepth := ADepth
  else
    FDepth := 10;

  FreeAll;
  Init;
  for I:= 1 to FDepth do
    Recurse;
  ShowRangeList;
end;

function Min(A, B: Double): Double;
begin
  if (A < B) then
    Result := A
  else
    Result := B
end;

function Max(A, B: Double): Double;
begin
  if (A > B) then
    Result := A
  else
    Result := B
end;

procedure TFractal1.SetStartRange(ARange: TRange);
var
  I: Integer;
begin
  FStartRange.head := Min(ARange.head, ARange.tail);
  FStartRange.tail := Max(ARange.head, ARange.tail);
  FreeAll;
  Init;
  for I:= 1 to FDepth do
    Recurse;
  ShowRangeList;
end;

procedure TFractal1.SetViewRange(ARange: TRange);
begin
  FViewRange.head := Min(ARange.head, ARange.tail);
  FViewRange.tail := Max(ARange.head, ARange.tail);
  if (FViewRange.head = FViewRange.tail) then
    FViewRange.tail := FViewRange.head + 1;
  ShowRangeList;
end;

end.
